use std::time::Instant;

use aoc::util::parse::ParseExt;

struct Solution {
    year: u32,
    day: u32,
    input: &'static str,
    run: fn(&str) -> (String, String),
}

macro_rules! solution {
    ($year:tt, $day:tt) => {
        Solution {
            year: (&stringify!($year)).parse_u32(),
            day: (&stringify!($day)).parse_u32(),
            input: include_str!(concat![
                "../input/",
                stringify!($year),
                "/",
                stringify!($day),
                ".txt"
            ]),
            run: |input: &str| {
                use aoc::$year::$day::*;
                let part1 = part1(&input).to_string();
                let part2 = part2(&input).to_string();
                (part1, part2)
            },
        }
    };
}

fn main() {
    let (year, day) = match std::env::args().nth(1) {
        Some(arg) => {
            let mut split = arg.split("::").map(|s| s.parse_u32());
            (split.next(), split.next())
        }
        None => (None, None),
    };
    let solutions: Vec<_> = solutions()
        .filter(|s| year == Some(s.year) || year.is_none())
        .filter(|s| day == Some(s.day) || day.is_none())
        .collect();

    for Solution {
        year,
        day,
        input,
        run,
    } in solutions
    {
        let start = Instant::now();
        let (aw1, aw2) = (run)(input);
        let elapsed = start.elapsed();
        println!("{year} Day {day:02} ({})", elapsed.as_micros());
        println!("    Part 1: {aw1}");
        println!("    Part 2: {aw2}");
    }
}

fn solutions() -> impl Iterator<Item = Solution> {
    std::iter::empty().chain(year2016())
}

fn year2016() -> Vec<Solution> {
    vec![solution!(year2016, day01)]
}
